class Persona:
	def __init__(self, nome, cognome):
		self.nome = nome
		self.cognome = cognome
		self.mail =""

# quando creo un professore devo specificare 
# la sua email.
#
class Professore(Persona):
	def __init__(self, nome, cognome, mail):
		Persona.__init__(self, nome, cognome)
		self.mail = mail
		self.ufficio = ""
		self.telefono = -1

class Studente(Persona):
	def __init__(self, nome, cognome, matricola, facolta):
		Persona.__init__(self, nome, cognome)
		self.facolta = facolta
		self.matricola = matricola
		# memorizza nomi esami (Stringa) e Esami (oggetti)
		self.dict_esami = {}

	def registraEsame(self, nomeEsame, voto):
		trovato = False
		esami = self.facolta.getListaEsami()
		
		for esame in esami:
			if nomeEsame == esame.nome and voto >= 18:
				self.dict_esami[nomeEsame] = voto
				trovato = True	
		if not trovato:
			print("Nella facolta", self.facolta.nome, "non c'e` l'esame di", nomeEsame, "oppure il voto", voto, "non e` valido")
		
	def getListaEsami(self):
		esami = []
		for esame in self.facolta.getListaEsami():
			esami.append(esame.nome)
		return esami

	def getEsamiSostenuti(self):
		esami = []
		for esame in self.dict_esami.keys():
			esami.append(esame)
		return esami

	def getEsitoEsami(self):
		es = dict()
		for nome in self.dict_esami.keys():
			es[nome] = self.dict_esami[nome]
		return es

class Facolta:
	# costringo a costruire una facolta con un nome e una lista di esami
	def __init__(self, nome, esami):
		self.nome = nome
		self.esami = esami
		self.studenti = []
		self.professori = []

	def aggiungiStudente(self, studente):
		self.studenti.append(studente)

	def aggiungiProfessore(self, professore):
		self.professori.append(professore)

	def getStudente(self, nome, cognome):
		for studente in self.studenti:
			if studente.nome == nome and studente.cognome == cognome:
				return studente
		# non trovato. None parola chiave che indica nessun oggetto
		return None

	def getStudenti(self):
		return self.studenti

	def getProfessori(self):
		return self.professori

	def getListaEsami(self):
		return self.esami

class Esame:
	def __init__(self, nome, crediti):
		self.nome = nome
		self.crediti = crediti
	
	def setCrediti(self, valore):
		# suppongo che ogni esame abbia almeno sei crediti e non piu` di 12
		# 
		if valore >= 4 and valore <= 12:
			self.crediti = valore
		else:
			print("Errore: crediti tra 4 e 12")

class Universita:
	def __init__(self, nome, luogo):
		self.nome = nome
		self.luogo = luogo
		self.facolta = {}
		self.__ultima_matricola = 0
	
		self.esami = dict()
		# costruisco l'universita` con un po' di facolta'...
		listaEsami = []
		for es in ["Analisi I", "Chimica", "Geometria", "Fondamenti di Informatica", "Fisica"]:
			listaEsami.append(Esame(es, 10))
		self.esami["Ingegneria"] = listaEsami

		listaEsami = [] # ripulisco e riutilizzo la lista
		for es in ["Storia dell'arte", "Scienza delle costruzioni", "Matematica", "Geometria"]:
			listaEsami.append(Esame(es, 8))
		self.esami["Architettura"] = listaEsami

		listaEsami = []
		for es in ["Diritto Privato", "Diritto Civile", "Storia del diritto", "Filosofia del Diritto"]:
			listaEsami.append(Esame(es, 8))
		self.esami["Giurisprudenza"] = listaEsami
	
		listaEsami = []
		for es in ["Storia della Filosofia", "Estetica", "Filosofia del Linguaggio", "Filosofia della Religione", "Filosofia Teoretica"]:
			listaEsami.append(Esame(es, 8))
		self.esami["Filosofia"] = listaEsami



		for nome in ("Ingegneria", "Architettura", "Giurisprudenza", "Filosofia"):
			f = Facolta(nome, self.esami[nome])
			self.facolta[nome] = f

	def getNomiFacolta(self):
		return self.facolta.keys()

	def getListaFacolta(self):
		return self.facolta.values()
	
	def iscriviStudente(self, nome, cognome, nomeFacolta):
	#	print("Iscrivo ", nome, cognome, "in facolta'", nomeFacolta)
		if nomeFacolta in self.facolta.keys():
			self.__ultima_matricola = self.__ultima_matricola + 1
			s = Studente(nome, cognome, self.__ultima_matricola, self.facolta[nomeFacolta])
			fac = self.facolta[nomeFacolta]
	#		print("Iscrivo ", s.nome, s.cognome, "facolta ", fac, fac.nome)
			fac.aggiungiStudente(s)
		else:
			print("In questa universita' non c'e' la facolta' di", nomeFacolta)

	def getStudente(self, nome, cognome):
		for fac in self.facolta.values():
			s = fac.getStudente(nome, cognome)
			if s != None:
				return s
		# non trovato
		return s # sara' None

	def getFacolta(self, nome):
		return self.facolta[nome]
# ----
# 
# Funzioni utilizzatrici delle classi create
#
# Funzione che usa le classi create per iscrivere alcuni studenti	
def PopolaUniversita():
	
	u = Universita("Universita` degli Studi di Trieste", "Trieste")

	# iscriviamo un po' di studenti in varie facolta`
	u.iscriviStudente("Mario", "Resistenza", "Ingegneria")
	u.iscriviStudente("Luca", "Transistore", "Ingegneria")
	u.iscriviStudente("Lucia", "Navali", "Ingegneria")
	u.iscriviStudente("Marta", "Industria", "Ingegneria")
	u.iscriviStudente("Carlo", "Valvola", "Ingegneria")
	u.iscriviStudente("Pippo", "Resistenza", "Ingegneria")

	u.iscriviStudente("Paolo", "Croce", "Filosofia")
	u.iscriviStudente("Benedetto", "Croce", "Filosofia")
	u.iscriviStudente("Federico", "Nietzsche", "Filosofia")
	u.iscriviStudente("Giordano", "Bruno", "Filosofia")
	u.iscriviStudente("Pippo", "Campanella", "Filosofia")


	return u

# funzione che recupera alcuni studenti dall'istanza di Universita passata come parametro
# e registra alcuni esami
def RegistraUnPoDiEsami(u):
	paoloCroce = u.getStudente("Paolo", "Croce")
	paoloCroce.registraEsame("Storia della Filosofia", 18)
	
	benedettoCroce = u.getStudente("Benedetto", "Croce")
	benedettoCroce.registraEsame("Storia della Filosofia", 30)

	marioRes = u.getStudente("Mario", "Resistenza")
	marioRes.registraEsame("Fondamenti di Informatica", 22)


